/**
 * Created by ofersarid on 09/05/2018.
 */
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const pkg = require('./package.json');

const scssExtract = new ExtractTextPlugin('[name]-[hash].css');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');

const resolvePlugins = (env) => {
  const plugins = [
    scssExtract,
    new HtmlWebpackPlugin({ template: './src/index.html' }),
  ];
  return env.is === 'prod' ? plugins.concat([
    new CleanWebpackPlugin(['dist']),
    new CopyWebpackPlugin([
      { from: '**/*.scss', to: 'scss', context: './src' },
      { from: '**/*.*', to: 'assets', context: './assets' },
    ]),
    new UglifyJsPlugin(),
  ]) : plugins;
};

module.exports = env => {
  console.log(`Environment = ${env.is}`);
  console.log('-------------------------------');
  return ({
    mode: env.is === 'dev' ? 'development' : 'production',
    entry: './src/index.js',
    output: {
      path: path.resolve(__dirname, 'dist'),
      // publicPath: '/assets',
      filename: 'app.bundle.js',
    },
    devtool: 'inline-source-map',
    devServer: {
      contentBase: './dist',
      compress: true,
      port: 3001,
    },
    externals: env.isMicro ? Object.keys(pkg.dependencies) : [],
    module: {
      rules: [
        {
          test: /\.jsx?$/,
          exclude: /(node_modules|bower_components)/,
          loader: 'babel-loader',
        },
        {
          test: /\.scss$/,
          use: scssExtract.extract({
            fallback: 'style-loader',
            use: ['css-loader', 'sass-loader'],
          }),
          include: path.resolve(__dirname, './src'),
        },
        {
          test: /\.(gif|jpg|pngs)$/,
          loader: 'url-loader?limit=10000',
          include: path.resolve(__dirname, './src'),
        },
        {
          test: /\.css$/,
          use: scssExtract.extract({
            fallback: 'style-loader',
            use: ['css-loader'],
          }),
          include: path.resolve(__dirname, './src'),
        },
        {
          test: /\.(woff2?|ttf|svg|eot|jpg|png|gif)?(\?.+)?$/,
          use: [{ loader: 'file-loader' }],
        },
        {
          test: /\.html$/,
          use: 'raw-loader',
        },
      ],
    },
    plugins: resolvePlugins(env),
  });
};
